  
#define TINY_GSM_MODEM_ESP8266

#include <Adafruit_CircuitPlayground.h>
#include "ComputerConnection.h"
#include "BlynkCode.h"
#include "MQTT.h"
#include <string>
#include <iostream>

// The Rube Goldberg machine works as follows
//The switchboard is activated when it is exposed to light (as the sun comes up in the morning)
//The Arduino code then connects to wifi and sends a GET request to the API APIweather.com. 
//The Arduino code checks if the current temperature (received from the api) is greater than 25 degrees.\
//Next, the Arduino code connects to Blynk, sending it the 'current temperature'.
//Blynk uses a webhook to connect to Make.com scenario 1.
//The Make.com scenario sends an email saying "Water your plant".
//This email activates the second Make.com scenario which is triggered when it receives the previous email. THis then triggers the Make.com scenario2. 
//The make.com scenario 2 then sends a discord message saying to water your plant.
//This discord message triggers a third make.com scenario to do 2 things in paralell.
//a) The Make.com Scenario connects back to Blynk and back to the Arduino Code. 
//The arduino code then connects to NodeRed using Adafruit through the MQTT server. The NodeRed causes the text "The current temperature is {current_temp}
//to be written to a textfile.
//b) The Make.com scenario connects to a 4th Make.com scenario using a webhook which causes an IOS notification to be sent to a cellphone.
//This will cause the cellphone to vibrate and knock over a set of dominos which will knock over a cup of water which will water the plant. 

//Online Tools:
//Arduino code, Make.com, Adafruit, Blynk, NodeRed
//Physcial Tools:
//Playground switchboard, cellphone, dominos, plant, cup of water. 
//Video Link:
//pdf Link:
//Instrutable Link:

//Created By: Shachar Weinmann 324974443 
//Liat Shear 805804



//The dish to parse
String dishToParse = "";

void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);
  CircuitPlayground.begin();
  computerConnectionSetup();
  
}

bool started = false;
bool MQTTBool = false;
bool finished = false;
float tempData = -1;
void loop() {
  int lightSensorValue = CircuitPlayground.lightSensor(); // get Light Sensor Value
  // If serial has an input dish, parse it here:
  if (lightSensorValue > 30)
  {
    if (!started) {
           //Wait for opening of the monitor (for testing)
           delay(2000);
           //get data
           tempData = getTempData();
           Serial.println(tempData);
           if (tempData >= 25) {
              //Begin Blynk and send tempData to make via WebHook
               blynkBegin();
               blynkRunTemp(tempData);
           }
        started = true;
      }
    
    Blynk.run();
  
    //MQTT LOOP
    if (MQTTBool) {
      if (!mqtt.connected()) {
        uint32_t t = millis();
        if (t - lastReconnectAttempt > 10000L) {
          lastReconnectAttempt = t;
          if (mqttConnect()) {
            lastReconnectAttempt = 0;
          }
        }
        delay(100);
        return;
      }
      mqtt.loop();
      // disconnects occur otherwise
      delay(1000);
      if (finished == false) {
        finished = true;
        Serial.println("publishing temperatures");
        //float ftemp = CircuitPlayground.temperature();
        tempStr = String(tempData);
        tempStr.toCharArray(temp, tempStr.length() + 1);
        mqtt.publish(temperatureTopic, temp);
      }
    }
    mqtt.loop();
  }
}

BLYNK_WRITE(V1) {
  //Setup MQTT
  MQTTSetup();
  //MQTT LOOP
  MQTTBool = true;
  
  // Circle lights in red and play alarm sound
    for (int i = 0; i < 10; i++) {
      CircuitPlayground.setPixelColor(i, 255, 0, 0); // Set red color
      delay(50);
    }
    CircuitPlayground.clearPixels();
    CircuitPlayground.playTone(100, 500);
    delay(1000);
}
